#****************************************************************************
#**
#**  File     :  /cdimage/units/XRL0110/XRL0110_script.lua
#**  Author(s):  John Comes, David Tomandl
#**
#**  Summary  :  Cybran Mobile Mortar Script
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local CWalkingLandUnit = import('/lua/cybranunits.lua').CWalkingLandUnit
local CybranWeaponsFile = import('/lua/cybranweapons.lua')
local CIFGrenadeWeapon = CybranWeaponsFile.CIFGrenadeWeapon

local EffectUtils = import('/lua/effectutilities.lua')
local CDFParticleCannonWeapon = import('/lua/cybranweapons.lua').CDFParticleCannonWeapon
local Effects = import('/lua/effecttemplates.lua')
local CLandUnit = import('/lua/cybranunits.lua').CLandUnit
local CIFMissileLoaWeapon = import('/lua/cybranweapons.lua').CIFMissileLoaWeapon
local CDFElectronBolterWeapon = CybranWeaponsFile.CDFElectronBolterWeapon
local CDFLaserHeavyWeapon = CybranWeaponsFile.CDFLaserHeavyWeapon


BRL0110 = Class(CWalkingLandUnit) {
    DestructionTicks = 400,

    Weapons = {
    	DummyWeapon = Class(CIFGrenadeWeapon) {},
        FlameGun = Class(CIFGrenadeWeapon) {},
	MissileRack = Class(CIFMissileLoaWeapon) {},
	Bolter = Class(CDFElectronBolterWeapon) {},
        GatlingCannon = Class(CDFLaserHeavyWeapon) 
        {
            PlayFxWeaponPackSequence = function(self)
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(0)
                end
                self.ExhaustEffects = EffectUtils.CreateBoneEffects( self.unit, 'Gat_Muzzle', self.unit:GetArmy(), Effects.WeaponSteam01 )
                self.ExhaustEffects = EffectUtils.CreateBoneEffects( self.unit, 'Gat_Exhaust', self.unit:GetArmy(), Effects.WeaponSteam01 )
                CDFLaserHeavyWeapon.PlayFxWeaponPackSequence(self)
            end,
        
            PlayFxRackSalvoChargeSequence = function(self)
                if not self.SpinManip then 
                    self.SpinManip = CreateRotator(self.unit, 'Gat_Spinner', 'z', nil, 270, 180, 60)
                    self.unit.Trash:Add(self.SpinManip)
                end
                
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(-700)
                end
                CDFLaserHeavyWeapon.PlayFxRackSalvoChargeSequence(self)
            end,            
        
        },
		
    },
    
    OnStopBeingBuilt = function(self,builder,layer)
        	local wep = self:GetWeaponByLabel('FlameGun')
		local flamerspeed = wep:GetBlueprint().FlamerSpeedMult or 1.2
            	self:SetSpeedMult(flamerspeed)
        	self:SetTurnMult(flamerspeed)


        CWalkingLandUnit.OnStopBeingBuilt(self,builder,layer)
    end,
    

}

TypeClass = BRL0110

